/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/search/SlideUri.java,v 1.8.2.1 2004/02/05 16:05:09 mholz Exp $
 * $Revision: 1.8.2.1 $
 * $Date: 2004/02/05 16:05:09 $
 *
 * ====================================================================
 *
 * Copyright 1999-2002 The Apache Software Foundation 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package org.apache.slide.search;

/**
 * The root of an href in slide is the slide servlet. For an external href the
 * root is the server. Example: the host is localhost, the context is slide,
 * the slidePath is /mycoll/myfile.xml
 * http://localhost/slide/mycoll/myfile.xml
 *
 */
public class SlideUri {
    
    private String context;
    private String path;
    
    /**
     * Constructs a SlideUri. If the requestUri is null, paths are regarded
     * as being always slidePaths
     */
    public SlideUri (String requestUri) {
        if (requestUri == null) {
            this.context = null;
            this.path = "";
        }
        else {
            this.context = getContextOfRelPath(requestUri);
            this.path = getPathOfRelPath(requestUri);
        }
    }
    
    /**
     * Method getSlideUri. If the requestUri is null, relpath is regarded to be
     * already a slidePath, it must start with a "/"
     *
     * @param    relPath             a  String
     *
     * @return   a String
     *
     * @throws   InvalidScopeException
     *
     */
    public String getSlidePath (String relPath) throws InvalidScopeException {
        
        if (context == null) {
            if (!relPath.startsWith("/"))
                throw new InvalidScopeException ("absolute scope is required");
            return relPath;
        }
        
        StringBuffer sb = new StringBuffer ();
        if (relPath.startsWith("/")) {
            
            if (!context.equals ("")) {
                
                String relContext = getContextOfRelPath (relPath);
                if (!relContext.equals (context))
                    throw new InvalidScopeException ("Uri \"" + relPath + "\" does not refer to " + context
                                                         + ". If an absolute scope is used, it must start with \""
                                                         + context + "\"");
                
                relPath = getPathOfRelPath (relPath);
            }
        }
            
        else {
            sb.append (path);
            if (!path.endsWith("/"))
                sb.append ("/");
        }
        
        sb.append (relPath);
        
        return sb.toString();
    }
    
    /**
     * Method getContextPath
     *
     * @param    internalHref        a  String
     *
     * @return   a String
     * @deprecated
     */
    public String getContextPath (String slidePath) {
        if (context == null) {
            throw new RuntimeException ("getContextPath not allowed in this context");
        }
        return context + slidePath;
    }
    
    
    /**
     * Method getContextOfRelPath
     *
     * @param    relPath             a  String
     *
     * @return   a String
     *
     */
    private String getContextOfRelPath (String relPath) {
        String result = null;
        int posSlash = relPath.indexOf ('/', 1);
        if (posSlash == -1)
            result = relPath.substring (0);
        else
            result = relPath.substring (0, posSlash);
        
        return result;
    }
    
    /**
     * Method getPathOfRelPath
     *
     * @param    relPath             a  String
     *
     * @return   a String
     *
     */
    private String getPathOfRelPath (String relPath) {
        String result = null;
        int posSlash = relPath.indexOf ('/', 1);
        if (posSlash == -1)
            result = "";
        else
            result = relPath.substring (posSlash);
        
        return result;
    }
    
    
    
    
    
}

